@@ -40,19 +40,33 @@ typedef enum {
 	NETWORK_DRUID_FINISH
 } NetworkDruidPages;
 
+typedef enum {
+	DRUID_START_PAGE,
+	DRUID_NORMAL_PAGE,
+	DRUID_FINISH_PAGE
+} NetworkDruidPageType;
+	
+
 typedef struct {
+	GtkWidget *window;
 	GstTool *tool;
 	GstConnection *cxn;
 
 	gboolean fixed_type; /* if the druid has a fixed GstConnectionType that cannot change */
 	gint current_page;
+
+	gint npages;
+	gchar *first_page_title;
+	gchar *title;
+	gchar *finish_title;
 } NetworkDruidData;
 
-void network_druid_new (GnomeDruid*, GstTool*, GstConnectionType);
+void network_druid_new (GnomeDruid*, GtkWidget*, GstTool*, GstConnectionType);
 void network_druid_clear (GnomeDruid*, gboolean);
 GstConnection* network_druid_get_connection_data (GnomeDruid*);
 void network_druid_check_page (GnomeDruid*, NetworkDruidPages);
 gboolean network_druid_set_page_next (GnomeDruid*);
 gboolean network_druid_set_page_back (GnomeDruid*);
+void network_druid_set_window_title (GnomeDruid*);
 
 #endif /* NETWORK_DRUID_H */
